'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (fixture, config, options) {
  // Compiler Config
  config = {
    devtool: config.devtool || false,
    context: config.context || _path2.default.resolve(process.cwd(), 'test/fixtures'),
    entry: config.entry || `./${fixture}`,
    output: output(config),
    module: modules(config),
    plugins: plugins(config)
  };

  // Compiler Mode
  config = mode(config);

  // Compiler Optimizations
  config = optimize(config);

  // Compiler Options
  options = Object.assign({ output: false }, options);

  if (options.output) {
    _del2.default.sync(config.output.path);
  }

  const compiler = (0, _webpack2.default)(config);

  if (!options.output) {
    compiler.outputFileSystem = new _memoryFs2.default();
  }

  return new Promise((resolve, reject) => compiler.run((err, stats) => {
    if (err) reject(err);

    resolve(stats);
  }));
};

var _del = require('del');

var _del2 = _interopRequireDefault(_del);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _memoryFs = require('memory-fs');

var _memoryFs2 = _interopRequireDefault(_memoryFs);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _package = require('webpack/package.json');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const mode = config => {
  if (Number(_package.version[0]) >= 4) {
    config.mode = config.mode || 'development';
  }

  return config;
}; /* eslint-disable
   import/order,
   no-param-reassign
   */


const optimize = config => {
  if (Number(_package.version[0]) >= 4) {
    config.optimization = config.optimization || {
      namedChunks: true,
      namedModules: true,
      runtimeChunk: true,
      occurrenceOrder: true
    };
  }

  return config;
};

const modules = config => {
  return {
    rules: config.rules ? config.rules : config.loader ? [{
      test: config.loader.test || /\.txt$/,
      use: {
        loader: _path2.default.resolve(process.cwd(), 'src'),
        options: config.loader.options || {}
      }
    }] : []
  };
};

const plugins = config => [Number(_package.version[0]) >= 4 ? false : new _webpack2.default.optimize.CommonsChunkPlugin({
  names: ['runtime'],
  minChunks: Infinity
})].concat(config.plugins || []).filter(Boolean);

const output = config => {
  return {
    path: _path2.default.resolve(process.cwd(), `test/outputs/${config.output ? config.output : ''}`),
    // publicPath: 'assets/',
    filename: '[name].js',
    chunkFilename: '[name].js'
  };
};